/*
Copyright 2021 The Knative Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package versioned

import (
	"fmt"

	discovery "k8s.io/client-go/discovery"
	rest "k8s.io/client-go/rest"
	flowcontrol "k8s.io/client-go/util/flowcontrol"
	bindingsv1beta1 "knative.dev/eventing-kafka/pkg/client/clientset/versioned/typed/bindings/v1beta1"
	kafkav1alpha1 "knative.dev/eventing-kafka/pkg/client/clientset/versioned/typed/kafka/v1alpha1"
	messagingv1beta1 "knative.dev/eventing-kafka/pkg/client/clientset/versioned/typed/messaging/v1beta1"
	sourcesv1beta1 "knative.dev/eventing-kafka/pkg/client/clientset/versioned/typed/sources/v1beta1"
)

type Interface interface {
	Discovery() discovery.DiscoveryInterface
	BindingsV1beta1() bindingsv1beta1.BindingsV1beta1Interface
	KafkaV1alpha1() kafkav1alpha1.KafkaV1alpha1Interface
	MessagingV1beta1() messagingv1beta1.MessagingV1beta1Interface
	SourcesV1beta1() sourcesv1beta1.SourcesV1beta1Interface
}

// Clientset contains the clients for groups. Each group has exactly one
// version included in a Clientset.
type Clientset struct {
	*discovery.DiscoveryClient
	bindingsV1beta1  *bindingsv1beta1.BindingsV1beta1Client
	kafkaV1alpha1    *kafkav1alpha1.KafkaV1alpha1Client
	messagingV1beta1 *messagingv1beta1.MessagingV1beta1Client
	sourcesV1beta1   *sourcesv1beta1.SourcesV1beta1Client
}

// BindingsV1beta1 retrieves the BindingsV1beta1Client
func (c *Clientset) BindingsV1beta1() bindingsv1beta1.BindingsV1beta1Interface {
	return c.bindingsV1beta1
}

// KafkaV1alpha1 retrieves the KafkaV1alpha1Client
func (c *Clientset) KafkaV1alpha1() kafkav1alpha1.KafkaV1alpha1Interface {
	return c.kafkaV1alpha1
}

// MessagingV1beta1 retrieves the MessagingV1beta1Client
func (c *Clientset) MessagingV1beta1() messagingv1beta1.MessagingV1beta1Interface {
	return c.messagingV1beta1
}

// SourcesV1beta1 retrieves the SourcesV1beta1Client
func (c *Clientset) SourcesV1beta1() sourcesv1beta1.SourcesV1beta1Interface {
	return c.sourcesV1beta1
}

// Discovery retrieves the DiscoveryClient
func (c *Clientset) Discovery() discovery.DiscoveryInterface {
	if c == nil {
		return nil
	}
	return c.DiscoveryClient
}

// NewForConfig creates a new Clientset for the given config.
// If config's RateLimiter is not set and QPS and Burst are acceptable,
// NewForConfig will generate a rate-limiter in configShallowCopy.
func NewForConfig(c *rest.Config) (*Clientset, error) {
	configShallowCopy := *c
	if configShallowCopy.RateLimiter == nil && configShallowCopy.QPS > 0 {
		if configShallowCopy.Burst <= 0 {
			return nil, fmt.Errorf("burst is required to be greater than 0 when RateLimiter is not set and QPS is set to greater than 0")
		}
		configShallowCopy.RateLimiter = flowcontrol.NewTokenBucketRateLimiter(configShallowCopy.QPS, configShallowCopy.Burst)
	}
	var cs Clientset
	var err error
	cs.bindingsV1beta1, err = bindingsv1beta1.NewForConfig(&configShallowCopy)
	if err != nil {
		return nil, err
	}
	cs.kafkaV1alpha1, err = kafkav1alpha1.NewForConfig(&configShallowCopy)
	if err != nil {
		return nil, err
	}
	cs.messagingV1beta1, err = messagingv1beta1.NewForConfig(&configShallowCopy)
	if err != nil {
		return nil, err
	}
	cs.sourcesV1beta1, err = sourcesv1beta1.NewForConfig(&configShallowCopy)
	if err != nil {
		return nil, err
	}

	cs.DiscoveryClient, err = discovery.NewDiscoveryClientForConfig(&configShallowCopy)
	if err != nil {
		return nil, err
	}
	return &cs, nil
}

// NewForConfigOrDie creates a new Clientset for the given config and
// panics if there is an error in the config.
func NewForConfigOrDie(c *rest.Config) *Clientset {
	var cs Clientset
	cs.bindingsV1beta1 = bindingsv1beta1.NewForConfigOrDie(c)
	cs.kafkaV1alpha1 = kafkav1alpha1.NewForConfigOrDie(c)
	cs.messagingV1beta1 = messagingv1beta1.NewForConfigOrDie(c)
	cs.sourcesV1beta1 = sourcesv1beta1.NewForConfigOrDie(c)

	cs.DiscoveryClient = discovery.NewDiscoveryClientForConfigOrDie(c)
	return &cs
}

// New creates a new Clientset for the given RESTClient.
func New(c rest.Interface) *Clientset {
	var cs Clientset
	cs.bindingsV1beta1 = bindingsv1beta1.New(c)
	cs.kafkaV1alpha1 = kafkav1alpha1.New(c)
	cs.messagingV1beta1 = messagingv1beta1.New(c)
	cs.sourcesV1beta1 = sourcesv1beta1.New(c)

	cs.DiscoveryClient = discovery.NewDiscoveryClient(c)
	return &cs
}
